function [y] = network_128(t,x,u,p)

%latest change i did here was that i added L as an output of this model 

% in this function we combine three wells towards a manifold that enters a
% riser. each of the wells will have different properties of PI,Kpr,GOR,WC,
% and will recieve Pm from the riser. The riser in return will accept
% values of gas and liquid flows from each well and the total flow to
% complete the circuit.
import casadi.*


x_well1 = [x(1:3); x(12:14)];   % the fiirst 24 states are the ones taken into account 
x_well2 = [x(4:6); x(15:17)];
x_well3 = [x(7:9); x(18:20)];
x_riser = [x(10:11); x(21:23)];
Pm      =  x(24);    % has been changed



% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% % Definig all constants to each model PI,P_res,GOR, WC,K_pr
% GOR     = [0.15 ; 0.1 ; 0.15];     % for [well1 well2 well3]
% WC      = [0.15 ; 0.1 ; 0.05];     % for [well1 well2 well3]
% Kpr     = [2.8  ; 3.0 ; 3.2 ;3.4]; % for [well1 well2 well3 riser]
% 
% p=[GOR(1),WC(1),Kpr(1),...
%    GOR(2),WC(2),Kpr(2),...
%    GOR(3),WC(3),Kpr(3),...
%    Kpr(4)];
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


% Definig all constants to each model PI,P_res,GOR, WC,K_pr, (Deafult) 
GOR     = [p(1); p(4); p(7)]*100;         % for [well1 well2 well3]
PI      = [3    ; 2.5 ; 2];        % for [well1 well2 well3] 
P_res   = [140  ; 150 ; 160];      % for [well1 well2 well3]
WC      = [p(2); p(5); p(8)]*100;         % for [well1 well2 well3]
Kpr     = [p(3); p(6); p(9); p(10)];     % for [well1 well2 well3 riser]

% % Definig all constants to each model PI,P_res,GOR, WC,K_pr
% GOR     = [0.15 ; 0.1 ; 0.15];     % for [well1 well2 well3]
% PI      = [3    ; 2.5 ; 2];        % for [well1 well2 well3] 
% P_res   = [140  ; 150 ; 160];      % for [well1 well2 well3]
% WC      = [0.15 ; 0.1 ; 0.05];     % for [well1 well2 well3]
% Kpr     = [2.8  ; 3.0 ; 3.2 ;3.4]; % for [well1 well2 well3 riser]

% % Definig all parameters equal, use this only when all p are equal 
% GOR     = [p(1); p(4); p(7)]*100;         % for [well1 well2 well3]
% PI      = [3    ; 3 ; 3];        % for [well1 well2 well3] 
% P_res   = [140  ; 140 ; 140];      % for [well1 well2 well3]
% WC      = [p(2); p(5); p(8)]*100;         % for [well1 well2 well3]
% Kpr     = [p(3); p(6); p(9); p(10)];     % for [well1 well2 well3 riser]

% % Definig all parameters equal, 
% GOR     = [p(1); p(4); p(7)]*100;         % for [well1 well2 well3]
% PI      = [3    ; 3 ; 3];        % for [well1 well2 well3] 
% P_res   = [150  ; 150 ; 150];      % for [well1 well2 well3]
% WC      = [p(2); p(5); p(8)]*100;         % for [well1 well2 well3]
% Kpr     = [p(3); p(6); p(9); p(10)];     % for [well1 well2 well3 riser]

u_well1 = [u(1);u(2)];                 % for well 1
u_well2 = [u(3);u(4)];                 % for well 2 
u_well3 = [u(5);u(6)];                 % for well 3
u_riser = u(end);                      % for riser 

%%%%%%%%%%%%%%%%%
%calling well model so we could find the flows and connect it to our riser 
well1 = well128_model(t,x_well1,u_well1,Pm,P_res(1),PI(1),Kpr(1),GOR(1),WC(1));

well2 = well128_model(t,x_well2,u_well2,Pm,P_res(2),PI(2),Kpr(2),GOR(2),WC(2));

well3 = well128_model(t,x_well3,u_well3,Pm,P_res(3),PI(3),Kpr(3),GOR(3),WC(3));
%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%
% calling riser model to find Pm  pressure and connect with well models 
flow1= well1(7:8);
flow2= well2(7:8);
flow3= well3(7:8);

WC_r  = (flow1(2)*WC(1) +flow2(2)*WC(2) + flow3(2)*WC(3))/ (flow1(2)+flow2(2)+ flow3(2));

riser = Riser128_model(t,x_riser,u_riser,flow1,flow2,flow3,WC_r,Kpr(4));

f     = riser(6)- Pm;
%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%
% Organized outputs
diff = [well1(1:3); well2(1:3); well3(1:3); riser(1:2)]  ; % 11 xdots
alg  = [well1(4:6); well2(4:6); well3(4:6); riser(3:5);f] ;% 13 residuals 


measurements=[riser(7);well1(7:end);well2(7:end);well3(7:end)];



% y: 24 elements consisting of  all states (11), and fsolve variables (13) a simple vector

% measurements consisting of the following : 
% 1	w_l_out               Riser %1
% 2	  w_g_out
% 3	  w_l_out
% 4	  w_g_in
% 5	  P_a_t               WELL 1 %8
% 6	  P_a_b
% 7	  P_t_t
% 8	  P_t_b
% 9	  P_bh
% 10	  w_g_out
% 11	  w_l_out
% 12	  w_g_in
% 13	  P_a_t           WELL 2 %8 
% 14	  P_a_b
% 15	  P_t_t
% 16	  P_t_b
% 17	  P_bh
% 18	      w_g_out
% 19	      w_l_out
% 20	      w_g_in
% 21	      P_a_t       WELL 3 %8
% 22	      P_a_b
% 23	      P_t_t
% 24	      P_t_b
% 25	      P_bh


y    = [diff;alg;measurements]; 
end
