%% Run this section first please specify the parameters and make sure that
% reservoir pressures and parameters are adjusted as desired both in this
% section and in function network_128

clear 
clc
close all


% How long would you like to run the ODE ? 
ode_time=4000;

% Initial states please do not adjust
x0 = [3.1933    0.5850    4.4132    3.3977    0.5601    5.0101    3.1600  ...
      0.5846    4.2971    0.2646    1.2903   15.6427    1.0425       15   14.3927  ...
      1.1855       20      14.6892    1.0511     28      48.0037    8.5439   39.4598   48.0963]';

% initial control inputs can be left the same 
u0 = [1;0.5;1;0.5;1;0.5;1];

% % Definig all constants to each model PI,P_res,GOR, WC,K_pr (deafult)
% GOR     = [0.15 ; 0.1 ; 0.15];     % for [well1 well2 well3]
% WC      = [0.15 ; 0.1 ; 0.05];     % for [well1 well2 well3]
% Kpr     = [2.8  ; 3.0 ; 3.2 ;3.4]; % for [well1 well2 well3 riser]

% all parameters are equal test
GOR     = [0.15 ; 0.15 ; 0.15];     % for [well1 well2 well3]
WC      = [0.15 ; 0.15 ; 0.15];     % for [well1 well2 well3]
Kpr     = [2.8  ; 2.8 ; 2.8 ;3.4]; % for [well1 well2 well3 riser]

% preparing parameter inputs to network_128.  Do not adjust 
p=[GOR(1),WC(1),Kpr(1),...
   GOR(2),WC(2),Kpr(2),...
   GOR(3),WC(3),Kpr(3),...
   Kpr(4)];


% running the fsolve function
opts = optimset('TolFun',1e-10,'TolX',1e-10,'MaxFunEvals',150000,'MaxIter',50000);

ff = @(x) [eye(24,24) zeros(24,25)]*network_128(0,x,u0,p);
[xz,fval]  = fsolve(ff,x0,opts);


%%  Plotting: OCP using values from ODE 

% note those are done for parameters decided above on the basis of the
% fsolve above 
x2_well1=[];
x3_well1=[];
w_l_out_r=[];
w_l_out_well1=[];
w_l_out_well2=[];
w_l_out_well3=[];
pm=[];
P_t_top_well1=[];
P_bh_well1=[];
P_bh_well2=[];
P_bh_well3=[];

% new input step 
for i=0.05:0.05:1
u_ode = [1;i;1;i;1;i;1];


 % Run Ode15s
M=diag([ones(1,11) zeros(1,13)]);
options=odeset('Mass',M);

%  begin p !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
FF= @(t,x) [eye(24,24) zeros(24,25)]*network_128(0,x,u_ode,p);
%   end  p !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


tspan=0:1:ode_time-1; 
    % [t,xz] = ode15s(FF,tspan,[x0;z0],options);
[t,xz_test11] = ode15s(FF,tspan,xz,options);


outputs_append=[];

for n=1:1:ode_time
    % Passing extra parameters
panda=network_128(0,(xz_test11(n,:))',u_ode,p);
    % Removing [differentials;residuals] 
panda(1:24)=[];
    % Appending outputs 
outputs_append=[outputs_append,panda];
end


% transforming outputs append to ode x 25 matrix
outputs_append=outputs_append';

w_l_out_r=[w_l_out_r;outputs_append(end,1)];
w_l_out_well1=[w_l_out_well1;outputs_append(end,3)];
w_l_out_well2=[w_l_out_well2;outputs_append(end,11)];
w_l_out_well3=[w_l_out_well3;outputs_append(end,19)];
pm=[pm;xz_test11(end,end)];

x2_well1=[x2_well1;xz_test11(end,2)];
x3_well1=[x3_well1;xz_test11(end,3)];

P_t_top_well1=[P_t_top_well1;outputs_append(end,7)];
P_bh_well1=[P_bh_well1;outputs_append(end,9)];
P_bh_well2=[P_bh_well2;outputs_append(end,17)];
P_bh_well3=[P_bh_well3;outputs_append(end,25)];

end

valveposition=100*[0.05:0.05:1];
%    save('plant_ode_ocp','xz','valveposition','pm','P_bh_well1','xz_test11','w_l_out_r','w_l_out_well1','w_l_out_well2','w_l_out_well3','outputs_append','t','p','u0','x0')


%% Plotting oil productions and pressure 

u_ode = [1;0.7;1;0.7;1;0.7;1];


 % Run Ode15s
M=diag([ones(1,11) zeros(1,13)]);
options=odeset('Mass',M);

%  begin p !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
FF= @(t,x) [eye(24,24) zeros(24,25)]*network_128(0,x,u_ode,p);
%   end  p !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


tspan=0:1:ode_time-1; 
    % [t,xz] = ode15s(FF,tspan,[x0;z0],options);
[t,xz_ode] = ode15s(FF,tspan,xz,options);


outputs_append=[];

for n=1:1:ode_time
    % Passing extra parameters
panda=network_128(0,(xz_ode(n,:))',u_ode,p);
    % Removing [differentials;residuals] 
panda(1:24)=[];
    % Appending outputs 
outputs_append=[outputs_append,panda];
end


% transforming outputs append to ode x 25 matrix
outputs_append=outputs_append';
%save('plant_ode_step_response_1','xz','xz_ode','u_ode','outputs_append','t','p','u0','x0')
%save('plant_ode_step_response_3','xz','xz_ode','u_ode','outputs_append','t','p','u0','x0')


%save('plant_ode_parameterchange_GOR','xz','xz_ode','u_ode','outputs_append','t','p','u0','x0')

